; This program is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program.  If not, see <http://www.gnu.org/licenses/>.

; ________________________________________________
; |       Programma scheda didattica 8 led       |
; |______________________________________________|
; |   Src File                                   |
; |______________________________________________|
; |   Tipo      : ASM                            |
; |   Versione  : 1                              |
; |   Release   : 0                              |
; |   Autore    : Lain Walter                    |
; |   Data      : 28/12/06                       |
; |______________________________________________|

        LIST

    __CONFIG 0x2150
; Program code protection: OFF
; Data code protection: OFF
; LVP: OFF
; BODEN: ON
; MCLRE: Vcc
; FOSC2: INTRC
; PWRT: ON
; WDT: OFF
; FOSC1-0: RA6,RA7 = I/O
 

#include "P16F628A.inc"
#include "macro.inc"
#include "costanti.inc"

BANK_0
#include "variabili.inc"

;***************************************************************************
;                                   RESET
;***************************************************************************

 ORG RESET_VECTOR
        GOTO SYS_STARTUP


;***************************************************************************
;                                INTERRUPTS
;***************************************************************************

 ORG INTERRUPT_VECTOR

        PUSH                            ;
        BANKSET 0                       ;Bank0
        BTFSS PIR1,TMR1IF               ;Controlla flag IRQ Timer1
        GOTO INT_END                    ; non attivo, prosegue
        CLRF PIR1                       ; clear flags IRQ
        MOVLF c_TMR_TIME_H,TMR1H        ;reimposta durata timer1
        MOVLF c_TMR_TIME_L,TMR1L        ;
        CLRF PIR1                       ;
        BANKSET 1                       ;Bank1
        BSF PIE1,TMR1IE                 ;Abilita IRQ Timer1
        BANKSET 0                       ;Bank0
        BSF f_TIMER1_ON                 ;attiva flag timer1
        INCF b_TMR2,F                   ;incrementa contatore timer2
        CMPBL b_TMR2,c_TMR_TIME2        ;verifica se timer2 arrivato al valore 
        BC INT_END                      ; no, prosegue
        CLRF b_TMR2                     ; si, azzera contatore
        BSF f_TIMER2_ON                 ;attiva flag timer2
INT_END POP                             ;
        RETFIE


;***************************************************************************
;                             START-UP SISTEMA
;***************************************************************************
SYS_STARTUP:

        CLI                             ;Disabilita master IRQ
        CLRF PORTA                      ;Inizializza PORTA
        CLRF PORTB                      ;Inizializza PORTB
        MOVLF 0x07,CMCON                ;disabilita comparatori
        BANKSET 1                       ;Bank1
        MOVLF 0xB,PCON                  ;Setta osc. interno 4MHz
        MOVLF 0x80,OPTION_REG           ;
        MOVLF 0x3D,TRISA                ;Setta RA7,RA6,RA1 come uscite
        MOVLF 0xC0,TRISB                ;Setta tutto PORTB come uscita tranne RB6 e RB7
        BANKSET 0                       ;Bank0
        MOVLF 0x3F,PORTB                ;Spegne tutti i led
        MOVLF 0xC2,PORTA                ;
;-------------Avvio iniziale
        BCF c_RLED_PORT,c_RLED_PIN      ;attiva led "RUN"
        CLRF R3H                        ;attiva tutti i led
        CALL Port_Write                 ;scrive sui port
        MOVLF 0x04,R2L                  ;carica numero cicli avvio
;1 ciclo=1,25us
;loop: 1 ciclo + loop3 + 5 o 6 cicli = 2010306 / 2010307 cicli ~ 2s
LOOP    MOVLF 0x64,R1H                  ;carica contatore 1
;loop3: (loop2 + 3 cicli) * 100 = 2010300 cicli
LOOP3   MOVLF 0x64,R1L                  ;carica contatore 2
;loop2: (loop1 + 3 cicli) * 100 = 20100
LOOP2   MOVLF 0x42,R2H                  ;carica contatore 3
;loop1: 3 cicli * 66 = 198
LOOP1   DECFSZ R2H                      ;decrementa contatore 3
        GOTO LOOP1                      ; cicla finchè contatore 3 > 0
        DECFSZ R1L                      ;decrementa contatore 2
        GOTO LOOP2                      ; cicla finchè contatore 2 > 0
        DECFSZ R1H                      ;decrementa contatore 1
        GOTO LOOP3                      ; cicla finchè contatore 1 > 0
        BTFSS c_RLED_PORT,c_RLED_PIN    ;controlla stato led
        GOTO LED_CLR                    ; clear
LED_SET BCF c_RLED_PORT,c_RLED_PIN      ; set-->clear
        CLRF R3H                        ;
        CALL Port_Write                 ;scrive sui port
        GOTO LOP_END                    ;torna a inizio ciclo
LED_CLR BSF c_RLED_PORT,c_RLED_PIN      ; clear-->set
        MOVLF 0xFF,R3H                  ;
        CALL Port_Write                 ;scrive sui port
LOP_END DECFSZ R2L                      ;verifica se finito ciclo avvio
        GOTO LOOP                       ;no, torna a inizio ciclo
;-------------Fine avvio iniziale
        CALL Timer_Init                 ;Inizializza i timer
        CALL Sysvar_Init                ;Inizializza variabili
        BSF INTCON,PEIE                 ;Abilita IRQ periferiche
        SEI                             ;Abilita master IRQ
        BSF f_ST_P6                     ;imposta stato Prog6
        BSF f_CTR_RUN                   ;Set esecuzione programmi
        BSF f_CTR_P6                    ;

;***************************************************************************
;                           PROGRAMMA PRINCIPALE
;***************************************************************************
PROGRAM:

        BTFSS f_TIMER1_ON               ;Controlla se avvenuto IRQ timer1
        GOTO PROGRAM                    ; no, torna a inizio ciclo

;esecuzione con timer1
        BCF f_TIMER1_ON                 ;clear flag IRQ timer1
        CALL Keyb_Read                  ;Chiama sub lettura pulsanti
        CALL Keyb_Run                   ;Chiama sub gestione "start/stop"
        BTFSS f_CTR_RUN                 ;Controlla se deve eseguire
        GOTO PROGRAM                    ; no, torna a inizio ciclo
        BTFSS f_ST_P5                   ;controlla se programma 5 in esecuzione
        GOTO PR_TM4                     ; no, prosegue controlli
        CALL Prog5                      ; si, esegue "PWM" 
PR_TM4  BTFSS f_TIMER2_ON               ;Controlla se finito conteggio timer2
        GOTO PROGRAM                    ; no, torna a inizio ciclo

;esecuzione solo con timer2
        BCF f_TIMER2_ON                 ; si, clear flag IRQ timer2
        CALL Timer_Exec                 ;chiama sub di gestione programmi
        BTFSC f_CTR_P1                  ;controlla se deve chiamare PROG1
        CALL Prog1                      ;chiama sub
        BTFSC f_CTR_P2                  ;no, controlla se deve chiamare PROG2
        CALL Prog2                      ;chiama sub
        BTFSC f_CTR_P3                  ;no, controlla se deve chiamare PROG3
        CALL Prog3                      ;chiama sub
        BTFSC f_CTR_P4                  ;no, controlla se deve chiamare PROG4
        CALL Prog4                      ;chiama sub
        BTFSC f_CTR_P5                  ;no, controlla se deve chiamare PROG5
        CALL Prog5                      ;chiama sub
        BTFSC f_CTR_P6                  ;no, controlla se deve chiamare PROG6
        CALL Prog6                      ;chiama sub
        GOTO PROGRAM                    ;Torna a inizio ciclo

#include "programmi.inc"
#include "timer.inc"
#include "gestione.inc"
#include "keyboard.inc"

        END
